@@ -1,22 +1,21 @@
 """Run the generation process"""
 
+import sys
 from typing import Optional
 
-from . import model
-from . import dot
-from . import parser
 from . import dfd_dot_templates as TMPL
-from . import preprocessor
+from . import dot, model, parser, scanner
 
-def build(dfd_src: str, output_path: str, percent_zoom: int, bgcolor: str,
-          format: str, debug: bool,
-          snippet_by_name: dict[str, str] = None) -> None:
+
+def build(provenance: model.SourceLine, dfd_src: str, output_path: str,
+          percent_zoom: int, bgcolor: str, format: str, debug: bool,
+          snippet_by_name: model.SnippetByName = None) -> None:
     """Take a DFD source and build the final image or document"""
-    dfd_src = preprocessor.preprocess(output_path, dfd_src, debug, snippet_by_name)
-
-    statements = parser.parse(dfd_src)
+    lines = scanner.scan(provenance, dfd_src, snippet_by_name, debug)
+    statements = parser.parse(lines, debug)
     items_by_name = parser.check(statements)
-    statements = filter_items(statements)
+    statements = filter_statements(statements)
+
     gen = Generator()
     text = generate_dot(gen, statements, items_by_name)
     if debug:
@@ -31,8 +30,8 @@ class Generator:
 
     def append(self, line:str, statement: model.Statement) -> None:
         self.lines.append('')
-        source = model.pack(statement.line)
-        self.lines.append(f'/* {statement.line_nr}: {source} */')
+        text = model.pack(statement.source.text)
+        self.lines.append(f'/* {statement.source.line_nr}: {text} */')
         self.lines.append(line)
 
     def generate_item(self, item: model.Item) -> None:
@@ -58,8 +57,8 @@ class Generator:
         self.star_nr += 1
         return star_name
 
-    def generate_connection(self, conn: model.Connection,
-                            src_item: model.Item, dst_item: model.Item) -> None:
+    def generate_connection(self, conn: model.Connection, src_item: model.Item,
+                            dst_item: model.Item) -> None:
         text = conn.text or ''
         src_port = dst_port = ""
 
@@ -104,15 +103,13 @@ class Generator:
         return text
 
 
-def generate_dot(gen: Generator,
-                 statements: list[model.Statement],
+def generate_dot(gen: Generator, statements: model.Statements,
                  items_by_name: dict[str, model.Item]) -> str:
     """Iterate over statements and generate a dot source file"""
     def get_item(name: str) -> Optional[model.Item]:
         return None if name == '*' else items_by_name[name]
 
     for statement in statements:
-        error_prefix = model.mk_err_prefix_from(statement)
         match statement:
             case model.Item() as item:
                 gen.generate_item(item)
@@ -128,7 +125,7 @@ def generate_dot(gen: Generator,
     return gen.generate_dot_text()
 
 
-def filter_items(statements: list[model.Statement]) -> list[model.Statement]:
+def filter_statements(statements: model.Statements) -> model.Statements:
     # collect used items
     connected_items = set()
     for statement in statements:
@@ -147,4 +144,4 @@ def filter_items(statements: list[model.Statement]) -> list[model.Statement]:
                     continue
         new_statements.append(statement)
 
-    return new_statements
\ No newline at end of file
+    return new_statements
